Module: CPEE::Logging

Defined in:
lib/cpee-logging-xes-yaml/tools.rb,
lib/cpee-logging-xes-yaml/implementation.rb

Defined Under Namespace

Classes: DownloadYAML, Handler, HeaderAndFile, Overview, Subscription, Subscriptions, Topics

Constant Summary collapse

SERVER =
File.expand_path(File.join(__dir__,'implementation.xml'))

Class Method Summary collapse

Class Method Details

.doc(opts, topic, event_name, payload) ⇒ Object



162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
# File 'lib/cpee-logging-xes-yaml/tools.rb', line 162

def self::doc(opts,topic,event_name,payload)
  notification = JSON.parse(payload)
  instance = notification['instance-uuid']
  return unless instance

  log_dir = opts[:log_dir]

  instancenr = notification['instance']
  content = notification['content']
  activity = content['activity']
  activity = nil if activity == '' || activity&.strip == ''
  eid = content['eid']
  eid = nil if eid == '' || eid&.strip == ''
  parameters = content['parameters']
  receiving = content['received']

  event = {}
  event['concept:instance'] = instancenr
  event['concept:name'] = content['label'] if content['label']
  if content['endpoint']
    event['concept:endpoint'] = content['endpoint']
  end
  event['id:id'] = (activity.nil? && eid.nil?) ? 'ex-ante' : (activity || eid)
  event['cpee:activity'] = activity unless activity.nil?
  event['cpee:activity_uuid'] = content['activity-uuid'] if content['activity-uuid']
  event['cpee:instance'] = instance
  case event_name
    when 'calling'
      event['lifecycle:transition'] = 'start'
    when 'done'
      event['lifecycle:transition'] = 'complete'
    else
      event['lifecycle:transition'] = 'unknown'
  end
  event['cpee:lifecycle:transition'] = "#{topic}/#{event_name}"
  event['cpee:state'] = content['state'] if content['state']
  event['cpee:description'] = content['dslx'] if content['dslx']
  event['cpee:change_uuid'] = content['change_uuid'] if content['change_uuid']
  event['cpee:exposition'] = content['exposition'] if content['exposition']
  if topic == 'gateway' && event_name == 'decide'
    event['data'] = { 'condition' => content['code'], 'result' => content['condition'] }
  end
  unless parameters['arguments']&.nil?
    event['data'] = parameters['arguments']
  end if parameters
  if (event_name == 'modify' || event_name == 'change') && content['values']&.any?
    event['data'] = content['values'].map do |k,v|
      { 'name' => k, 'value' => v }
    end
  end

  if topic == 'task' && event_name == 'probe'
    rs = WEEL::ReadStructure.new(content['data'],{},{},{})
    rc = CPEE::Logging::extract_result(receiving) if receiving && !receiving.empty?
    te = event.dup

    content['probes'].each do |p|

      if p.dig('probe','extractor_type') == 'intrinsic'
        pid    = p.dig('probe','id')
        source = p.dig('probe','source')
        val = CPEE::Logging::extract_val(rs,p.dig('probe','extractor_code'),pid,nil) rescue nil
        # Do not add datastream entries if the dataprobes return nil
        if val != nil
          event['stream:datastream'] ||= []
          CPEE::Logging::merge_val(event['stream:datastream'],val,pid,source)
        end
        if event['stream:datastream'] && event['stream:datastream'].any?
          EM.defer do
            self::notify(
              opts,
              'stream',
              'extraction',
              notification.merge(
                'topic'=>'stream',
                'name'=>'extraction',
                'datastream'=>event['stream:datastream']
              ).to_json
            )
          end
        end
      end

      if receiving && !receiving.empty?
        if p.dig('probe','extractor_type') == 'extrinsic'
          pid    = p.dig('probe','id')
          source = p.dig('probe','source')
          val = CPEE::Logging::extract_val(rs,p.dig('probe','extractor_code'),pid,rc) rescue nil
          # Do not add datastream entries if the dataprobes return nil
          if val != nil
            te['stream:datastream'] ||= []
            CPEE::Logging::merge_val(te['stream:datastream'],val,pid,source)
          end
          if te['stream:datastream'] && te['stream:datastream'].any?
            te['cpee:lifecycle:transition'] = 'stream/data'
            File.open(File.join(log_dir,instance+'.xes.yaml'),'a') do |f|
              f << {'event' => te}.to_yaml
            end
            EM.defer do
              self::notify(
                opts,
                'stream',
                'extraction',
                notification.merge(
                  'topic'=>'stream',
                  'name'=>'extraction',
                  'datastream'=>te['stream:datastream']
                ).to_json
              )
            end
          end
        end
      end

    end

  end

  if receiving && !receiving.empty?
    event['data'] = receiving
  end
  if content['data'] && !content['data'].empty?
    event['data'] = content['data']
  end
  event['time:timestamp']= notification['timestamp'] || Time.now.xmlschema(4)
  File.open(File.join(log_dir,instance+'.xes.yaml'),'a') do |f|
    f << {'event' => event}.to_yaml
  end
end

.extract_result(result) ⇒ Object



117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
# File 'lib/cpee-logging-xes-yaml/tools.rb', line 117

def self::extract_result(result)
  ret = result.map do |res|
    if res['mimetype'].nil?
      res['value']
    elsif res['mimetype'] == 'application/json'
      JSON::parse(res['data'])
    elsif res['mimetype'] == 'application/xml' || res['mimetype'] == 'text/xml'
      XML::Smart::string(res['data']) rescue nil
    elsif res['mimetype'] == 'text/yaml'
      YAML::load(res['data']) rescue nil
    elsif res['mimetype'] == 'text/plain'
      t = res['data']
      if t.start_with?('<?xml version=')
        t = XML::Smart::string(t)
      else
        t = t.to_f if t == t.to_f.to_s
        t = t.to_i if t == t.to_i.to_s
      end
      t
    elsif res['mimetype'] == 'text/html'
      t = res['data']
      t = t.to_f if t == t.to_f.to_s
      t = t.to_i if t == t.to_i.to_s
      t
    else
      res['data']
    end
  end
  ret.length == 1 ? ret[0] : ret
end

.extract_val(rs, code, pid, result) ⇒ Object



148
149
150
151
152
# File 'lib/cpee-logging-xes-yaml/tools.rb', line 148

def self::extract_val(rs,code,pid,result)
  rs.instance_eval(code,'probe',1)
rescue => e
  e.backtrace[0].gsub(/(\w+):(\d+):in.*/,'Probe ' + pid + ' Line \2: ') + e.message
end

.forward(opts, topic, event_name, payload) ⇒ Object



154
155
156
157
158
159
160
# File 'lib/cpee-logging-xes-yaml/tools.rb', line 154

def self::forward(opts,topic,event_name,payload)
  if topic == 'state' && event_name == 'change'
    self::notify(opts,topic,event_name,payload)
  elsif topic == 'gateway' && event_name == 'join'
    self::notify(opts,topic,event_name,payload)
  end
end

.implementation(opts) ⇒ Object

}}}



179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
# File 'lib/cpee-logging-xes-yaml/implementation.rb', line 179

def self::implementation(opts)
  opts[:log_dir]           ||= File.expand_path(File.join(__dir__,'logs'))
  opts[:notifications_dir] ||= File.expand_path(File.join(__dir__,'notifications'))
  opts[:template]          ||= File.expand_path(File.join(__dir__,'template.xes_yaml'))
  opts[:topics]            ||= File.expand_path(File.join(__dir__,'topics.xml'))
  opts[:subscriptions]     =  {}

  opts[:sse_keepalive_frequency]    ||= 10

  Dir.glob(File.join(opts[:notifications_dir],'*','subscription.xml')).each do |f|
    XML::Smart::open_unprotected(f) do |doc|
      doc.register_namespace :p, 'http://riddl.org/ns/common-patterns/notifications-producer/2.0'
      doc.find('/p:subscription/p:topic').each do |t|
        t.find('p:event').each do |e|
          opts[:subscriptions][t.attributes['id']+'/'+e.text] ||= []
          opts[:subscriptions][t.attributes['id']+'/'+e.text] << doc.root.attributes['url']
        end
      end
    end
  end

  Proc.new do
    interface 'access' do
      on resource '[a-f0-9-]+.xes.yaml' do
        run DownloadYAML, opts if get
      end
      on resource '[a-f0-9-]+.xes.xml' do
        run DownloadXML, opts if get
      end
    end
    interface 'events' do
      run Handler, opts if post 'event'
    end
    interface 'notifications' do
      on resource 'notifications' do
        run Overview if get
        on resource 'topics' do
          run Topics, opts if get
        end
        on resource 'subscriptions' do
          run Subscriptions, opts if get
          run CreateSubscription, opts if post 'create_subscription'
          on resource do
            run Subscription, opts if get
            run UpdateSubscription, opts if put 'change_subscription'
            run DeleteSubscription, opts if delete
          end
        end
      end
    end
  end
end

.merge_val(target, val, tid, tso) ⇒ Object



98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
# File 'lib/cpee-logging-xes-yaml/tools.rb', line 98

def self::merge_val(target,val,tid,tso)
  if val.is_a? Stream
    val.source = tso if val.source.nil?
    target.push *val.to_list
  else
    tp = nil
    if val.is_a? StreamPoint
      tp = val
      tp.id = tid if tp.id.nil?
      tp.source = tso if tp.source.nil?
    else
      tp = StreamPoint.new(tid)
      tp.source =  tso
      tp.value = val
    end
    target << { 'stream:point' => tp.to_h }
  end
end

.notify(opts, topic, event_name, payload) ⇒ Object

Forward event to subscriptions



80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
# File 'lib/cpee-logging-xes-yaml/tools.rb', line 80

def self::notify(opts,topic,event_name,payload)
  opts[:subscriptions].each do |e,urls|
    if e == topic + '/' + event_name
      urls.each do |url|
        EM.defer do
          client = Riddl::Client.new(url)
          client.post [
            Riddl::Parameter::Simple::new('type','event'),
            Riddl::Parameter::Simple::new('topic',topic),
            Riddl::Parameter::Simple::new('event',event_name),
            Riddl::Parameter::Complex::new('notification','application/json',payload)
          ]
        end
      end
    end
  end
end